


!Number methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 12/13/2010 15:36'!
-\\+ aNumber

	"modulo with floor 1 (useful for indexation). 
	see #atWrap:
	"

	^ self - 1 \\ aNumber + 1

	! !

!Collection methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 12/3/2011 10:49'!
/// arg

	^ arg adaptToCollection: self andSend: #///! !

!Float methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 7/17/2012 19:55'!
/// aNumber

	^ self / aNumber! !

!Number methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 11/19/2009 13:31'!
/// aNumber

	"always return a Float or a SmallInteger. useful since a Fraction behaves much slower than a Float in most computations "

	^ self / aNumber asFloat! !

!Point methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 11/18/2009 18:15'!
/// arg 
	"Answer a Point that is the quotient of the receiver and arg."

	arg isPoint ifTrue: [^ (x /// arg x) @ (y /// arg y)].
	^ arg adaptToPoint: self andSend: #///! !

!Number methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 12/13/2010 15:30'!
\\+ aNumber

	"modulo offset by 1 (useful for indexation)"

	^ self \\ aNumber + 1

	! !

!Float methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 11/15/2011 15:42'!
asApproximateFractionWithDenominatorAtMost: maxDenominator

	| num1 denom1 num2 denom2 int frac newD temp |
	num1 := self asInteger.	"The first of two alternating numerators"
	denom1 := 1.		"The first of two alternating denominators"
	num2 := 1.		"The second numerator"
	denom2 := 0.		"The second denominator--will update"
	int := num1.		"The integer part of self"
	frac := self fractionPart.		"The fractional part of self"
	[frac = 0 or: [denom1 > maxDenominator] or: [denom2 > maxDenominator]]
		whileFalse: 
			["repeat while the fractional part is not zero and max order is not reached"
			newD := 1.0 / frac.			"Take reciprocal of the fractional part"
			int := newD asInteger.		"get the integer part of this"
			frac := newD fractionPart.	"and save the fractional part for next time"
			temp := num2.				"Get old numerator and save it"
			num2 := num1.				"Set second numerator to first"
			num1 := num1 * int + temp.	"Update first numerator"
			temp := denom2.				"Get old denominator and save it"
			denom2 := denom1.			"Set second denominator to first"
			denom1 := int * denom1 + temp.		"Update first denominator"].
	"If fractional part is zero, return the first ratio"
	denom1 = 1
		ifTrue: ["Am I really an Integer?"
				^ num1 "Yes, return Integer result"]
		ifFalse: ["Otherwise return Fraction result" 
				denom1 > maxDenominator 
					ifTrue: [^ Fraction numerator: num2 denominator: denom2].
				^ Fraction numerator: num1 denominator: denom1]! !

!Number methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 5/3/2011 21:10'!
asIntegerWhenClose

	| int |

	int _ self rounded.
	(int closeTo: self) ifTrue: [^ int].
	
	^ self! !

!Point methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 11/18/2009 18:15'!
closeTo: aPoint

	^ (x closeTo: aPoint x) and: [y closeTo: aPoint y]! !

!Float class methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 12/13/2010 13:29'!
halfTurn

	"http://tauday.com
	http://en.wikipedia.org/wiki/Turn_%28geometry%29"

	^ Pi! !

!Number methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 1/3/2011 15:40'!
isCloseToAnInteger

	^ self rounded closeTo: self! !

!Number methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 4/4/2010 14:49'!
negativeFlag

	^ self >= 0 ifTrue: [0] ifFalse: [1]! !

!Number methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 4/4/2010 14:49'!
positiveFlag

	^ self >= 0 ifTrue: [1] ifFalse: [0]! !

!Float class methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 12/13/2010 13:29'!
quarterTurn

	"http://tauday.com
	http://en.wikipedia.org/wiki/Turn_%28geometry%29"

	^ Halfpi! !

!Float class methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 12/13/2010 13:29'!
turn

	"http://tauday.com
	http://en.wikipedia.org/wiki/Turn_%28geometry%29"

	^ Twopi! !

!Number methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 11/18/2009 18:15'!
x

	"consistent with the usual graphic representation of imaginary numbers"

	^ self! !

!Number methodsFor: '*People-spfa-utils-maths' stamp: 'spfa 11/18/2009 18:15'!
y

	"consistent with the usual graphic representation of imaginary numbers"

	^ 0! !
